@@ -71,7 +71,7 @@ Win_GParted::Win_GParted( const std::vector<Glib::ustring> & user_devices )
 	
 	try
 	{
-		this ->set_icon_from_file( GNOME_ICONDIR "/gparted.png" ) ;
+		this ->set_default_icon_name( "gparted" ) ;
 	}
 	catch ( Glib::Exception & e )
 	{ 
@@ -141,9 +141,6 @@ void Win_GParted::init_menubar()
 	image = manage( new Gtk::Image( Gtk::Stock::HARDDISK, Gtk::ICON_SIZE_MENU ) );
 	menu ->items() .push_back( Gtk::Menu_Helpers::ImageMenuElem( _("_Devices"), *image ) ) ; 
 	
-	menu ->items() .push_back( Gtk::Menu_Helpers::SeparatorElem( ) );
-	menu ->items() .push_back( Gtk::Menu_Helpers::MenuElem(
-		_("_Show Features"), sigc::mem_fun( *this, &Win_GParted::menu_gparted_features ) ) );
 	menu ->items() .push_back( Gtk::Menu_Helpers::SeparatorElem( ) );
 	menu ->items() .push_back( Gtk::Menu_Helpers::StockMenuElem( 
 		Gtk::Stock::QUIT, sigc::mem_fun(*this, &Win_GParted::menu_gparted_quit) ) );
@@ -167,7 +164,7 @@ void Win_GParted::init_menubar()
 		* manage( new Gtk::Image( Gtk::Stock::APPLY, Gtk::ICON_SIZE_MENU ) ), 
 		sigc::mem_fun(*this, &Win_GParted::activate_apply) ) );
 	menubar_main .items() .push_back( Gtk::Menu_Helpers::MenuElem( _("_Edit"), *menu ) );
-	
+
 	//view
 	menu = manage( new Gtk::Menu() ) ;
 	menu ->items() .push_back( Gtk::Menu_Helpers::CheckMenuElem(
@@ -175,21 +172,29 @@ void Win_GParted::init_menubar()
 	menu ->items() .push_back( Gtk::Menu_Helpers::CheckMenuElem( 
 		_("Pending _Operations"), sigc::mem_fun(*this, &Win_GParted::menu_view_operations) ) );
 	menubar_main .items() .push_back( Gtk::Menu_Helpers::MenuElem( _("_View"), *menu ) );
-	
+
+	menu ->items() .push_back( Gtk::Menu_Helpers::SeparatorElem( ) );
+	menu ->items() .push_back( Gtk::Menu_Helpers::MenuElem(
+		_("_File System Support"), sigc::mem_fun( *this, &Win_GParted::menu_gparted_features ) ) );
+
 	//device
 	menu = manage( new Gtk::Menu() ) ;
 	menu ->items() .push_back( Gtk::Menu_Helpers::MenuElem( Glib::ustring( _("_Create Partition Table") ) + " ...",
 								sigc::mem_fun(*this, &Win_GParted::activate_disklabel) ) );
 	menubar_main .items() .push_back( Gtk::Menu_Helpers::MenuElem( _("_Device"), *menu ) );
-		
+
 	//partition
 	init_partition_menu() ;
 	menubar_main .items() .push_back( Gtk::Menu_Helpers::MenuElem( _("_Partition"), menu_partition ) );
-	
+
 	//help
 	menu = manage( new Gtk::Menu() ) ;
-	menu ->items() .push_back(Gtk::Menu_Helpers::StockMenuElem(
-		Gtk::Stock::HELP, sigc::mem_fun(*this, &Win_GParted::menu_help_contents) ) );
+	menu ->items() .push_back( Gtk::Menu_Helpers::ImageMenuElem( 
+		_("_Contents"), 
+		Gtk::AccelKey("F1"),
+		* manage( new Gtk::Image( Gtk::Stock::HELP, Gtk::ICON_SIZE_MENU ) ), 
+		sigc::mem_fun(*this, &Win_GParted::menu_help_contents) ) );
+	menu ->items() .push_back( Gtk::Menu_Helpers::SeparatorElem( ) );
 	menu ->items() .push_back( Gtk::Menu_Helpers::StockMenuElem(
 		Gtk::Stock::ABOUT, sigc::mem_fun(*this, &Win_GParted::menu_help_about) ) );
 
@@ -218,7 +223,16 @@ void Win_GParted::init_toolbar()
 	
 	//RESIZE/MOVE
 	image = manage( new Gtk::Image( Gtk::Stock::GOTO_LAST, Gtk::ICON_SIZE_BUTTON ) );
-	toolbutton = Gtk::manage(new Gtk::ToolButton( *image, _("Resize/Move") ));
+	Glib::ustring str_temp = _("Resize/Move") ;
+	//Condition string split and Undo button.
+	//  for longer translated string, split string in two and skip the Undo button to permit full toolbar to display
+	//  FIXME:  Is there a better way to do this, perhaps without the conditional?  At the moment this seems to be the best compromise.
+	bool display_undo = true ;
+	if( str_temp .length() > 14 ) {
+		str_temp .replace( str_temp .find( "/" ), 1, "\n/" ) ;
+		display_undo = false ;
+	}
+	toolbutton = Gtk::manage(new Gtk::ToolButton( *image, str_temp ));
 	toolbutton ->signal_clicked().connect( sigc::mem_fun(*this, &Win_GParted::activate_resize) );
 	toolbar_main.append(*toolbutton);
 	TOOLBAR_RESIZE_MOVE = index++ ;
@@ -241,12 +255,15 @@ void Win_GParted::init_toolbar()
 	index++ ;
 	
 	//UNDO and APPLY
-	toolbutton = Gtk::manage(new Gtk::ToolButton(Gtk::Stock::UNDO));
-	toolbutton ->signal_clicked().connect( sigc::mem_fun(*this, &Win_GParted::activate_undo) );
-	toolbar_main.append(*toolbutton);
-	TOOLBAR_UNDO = index++ ;
-	toolbutton ->set_sensitive( false );
-	toolbutton ->set_tooltip(tooltips, _("Undo Last Operation") );		
+	if ( display_undo ) {
+		//Undo button is displayed only if translated language "Resize/Move" is not too long.  See above setting of this condition.
+		toolbutton = Gtk::manage(new Gtk::ToolButton(Gtk::Stock::UNDO));
+		toolbutton ->signal_clicked().connect( sigc::mem_fun(*this, &Win_GParted::activate_undo) );
+		toolbar_main.append(*toolbutton);
+		TOOLBAR_UNDO = index++ ;
+		toolbutton ->set_sensitive( false );
+		toolbutton ->set_tooltip(tooltips, _("Undo Last Operation") );
+	}
 	
 	toolbutton = Gtk::manage(new Gtk::ToolButton(Gtk::Stock::APPLY));
 	toolbutton ->signal_clicked().connect( sigc::mem_fun(*this, &Win_GParted::activate_apply) );
@@ -310,7 +327,7 @@ void Win_GParted::init_partition_menu()
 	index++ ;
 	
 	image = manage( new Gtk::Image( Gtk::Stock::CONVERT, Gtk::ICON_SIZE_MENU ) );
-	/*TO TRANSLATORS: menuitem which holds a submenu with filesystems.. */
+	/*TO TRANSLATORS: menuitem which holds a submenu with file systems.. */
 	menu_partition .items() .push_back(
 			Gtk::Menu_Helpers::ImageMenuElem( _("_Format to"),
 							  *image,
@@ -325,7 +342,7 @@ void Win_GParted::init_partition_menu()
 						     sigc::mem_fun( *this, &Win_GParted::toggle_swap_mount_state ) ) );
 	MENU_TOGGLE_MOUNT_SWAP = index++ ;
 		
-	/*TO TRANSLATORS: menuitem which holds a submenu with mountpoints.. */
+	/*TO TRANSLATORS: menuitem which holds a submenu with mount points.. */
 	menu_partition .items() .push_back(
 			Gtk::Menu_Helpers::MenuElem( _("_Mount on"), * manage( new Gtk::Menu() ) ) ) ;
 	MENU_MOUNT = index++ ;
@@ -570,6 +587,10 @@ void Win_GParted::show_pulsebar( const Glib::ustring & status_message )
 			Gtk::Main::iteration();
 		
 		usleep( 10000 );
+
+		Glib::ustring tmp_msg = gparted_core .get_thread_status_message() ;
+		if ( tmp_msg != "" )
+			statusbar .push( tmp_msg ) ;
 	}
 	
 	thread ->join() ;
@@ -667,10 +688,13 @@ void Win_GParted::Refresh_Visual()
 
 	//set new statusbartext
 	statusbar .pop() ;
-	if ( operations .size() != 1 )
-		statusbar .push( String::ucompose( _("%1 operations pending"), operations .size() ) );
-	else
-		statusbar .push( _( "1 operation pending" ) );
+	statusbar .push( String::ucompose( ngettext( "%1 operation pending"
+	                                           , "%1 operations pending"
+	                                           , operations .size()
+	                                           )
+	                                 , operations .size()
+	                                 )
+	               );
 		
 	if ( ! operations .size() ) 
 		allow_undo_clear_apply( false ) ;
@@ -724,11 +748,13 @@ bool Win_GParted::Quit_Check_Operations()
 					   Gtk::BUTTONS_NONE,
 					   true );
 
-		if ( operations .size() != 1 )
-			dialog .set_secondary_text( String::ucompose( _("%1 operations are currently pending."), 
-							  	      operations .size() ) ) ;
-		else
-			dialog .set_secondary_text( _("1 operation is currently pending.") ) ;
+		dialog .set_secondary_text( String::ucompose( ngettext( "%1 operation is currently pending."
+		                                                      , "%1 operations are currently pending."
+		                                                      , operations .size()
+		                                                      )
+		                                            , operations .size()
+		                                            )
+		                          ) ;
 	
 		dialog .add_button( Gtk::Stock::QUIT, Gtk::RESPONSE_CLOSE );
 		dialog .add_button( Gtk::Stock::CANCEL,Gtk::RESPONSE_CANCEL );
@@ -843,7 +869,7 @@ void Win_GParted::set_valid_operations()
 			allow_copy( true ) ;
 		
 		//only allow labelling of real partitions that support labelling
-		if ( selected_partition .status == GParted::STAT_REAL && fs .set_label )
+		if ( selected_partition .status == GParted::STAT_REAL && fs .write_label )
 			allow_label_partition( true ) ;
 
 		if ( selected_partition .get_mountpoints() .size() )
@@ -870,10 +896,11 @@ void Win_GParted::set_valid_operations()
 		if ( ! copied_partition .get_path() .empty() &&
 		     copied_partition .get_length() <= selected_partition .get_length() &&
 		     selected_partition .status == GParted::STAT_REAL &&
-		     copied_partition != selected_partition )
+		     copied_partition != selected_partition &&
+		     selected_partition .type == GParted::TYPE_UNALLOCATED )
 		     allow_paste( true ) ;
 
-		//see if we can somehow check/repair this filesystem....
+		//see if we can somehow check/repair this file system....
 		if ( fs .check && selected_partition .status == GParted::STAT_REAL )
 			allow_check( true ) ;
 	}
@@ -998,7 +1025,7 @@ void Win_GParted::menu_gparted_refresh_devices()
 	{
 		Gtk::MessageDialog dialog(
 			*this,
-			_("The kernel is unable to re-read the partitiontables on the following devices:") + 
+			_("The kernel is unable to re-read the partition tables on the following devices:") + 
 			Glib::build_path( "", readonly_paths ),
 			false,
 			Gtk::MESSAGE_WARNING,
@@ -1125,20 +1152,54 @@ void Win_GParted::menu_view_operations()
 		close_operationslist() ;
 }
 
-void Win_GParted::menu_help_contents()
+void Win_GParted::show_help_dialog (const char *link_id /* For context sensitive help */)
 {
-	Gtk::MessageDialog dialog( *this,
-				   _("Sorry, not yet implemented."),
-				   false,
-				   Gtk::MESSAGE_INFO,
-				   Gtk::BUTTONS_OK,
-				   true );
+/* Original concept for show_help_dialog is from file-roller project
+ * gtk-utils.c revision 1967, Tue Aug 7 06:18:42 2007 UTC
+ * See http://fileroller.sourceforge.net/
+ * 
+ * Modified to work with GParted by Curtis Gedak :-)
+ */
+	GError *err = NULL;
+	char *command;
+	const char *lang;
+	char *uri = NULL;
+	int i;
+	GdkScreen *gscreen;
 
-	dialog .set_secondary_text( _( "Please visit http://gparted.sf.net for more information and support.") ) ;
+	const char * const * langs = g_get_language_names ();
 
-	dialog .run();
+	for (i = 0; langs[i]; i++) {
+		lang = langs[i];
+		if (strchr (lang, '.')) {
+			continue;
+		}
+
+		uri = g_build_filename( GPARTED_DATADIR,
+								"/gnome/help/gparted/",
+								lang,
+								"/gparted.xml",
+								NULL );
+					
+		if (g_file_test (uri, G_FILE_TEST_EXISTS)) {
+                    break;
+		}
+	}
+	
+	if (link_id) {
+		command = g_strconcat ("gnome-open ghelp://", uri, "?", link_id, NULL);
+	} else {
+		command = g_strconcat ("gnome-open ghelp://", uri,  NULL);
+	}
+
+	gscreen = gdk_screen_get_default();
+	gdk_spawn_command_line_on_screen (gscreen, command, &err);
 }
 
+void Win_GParted::menu_help_contents()
+{
+	show_help_dialog( NULL);
+}
 
 void Win_GParted::menu_help_about()
 {
@@ -1148,7 +1209,7 @@ void Win_GParted::menu_help_about()
 	dialog .set_transient_for( *this ) ;
 	
 	dialog .set_name( _("GParted") ) ;
-	dialog .set_logo( this ->get_icon() ) ;
+	dialog .set_logo_icon_name( "gparted" ) ;
 	dialog .set_version( VERSION ) ;
 	dialog .set_comments( _( "GNOME Partition Editor" ) ) ;
 	dialog .set_copyright( "Copyright © 2004-2008 Bart Hakvoort" ) ;
@@ -1165,7 +1226,7 @@ void Win_GParted::menu_help_about()
 	strings .clear() ;
 
 	/*TO TRANSLATORS: your name(s) here please, if there are more translators put newlines (\n) between the names.
-	  It's a good idea to provide the url of your translationteam as well. Thanks! */
+	  It's a good idea to provide the url of your translation team as well. Thanks! */
 	Glib::ustring str_credits = _("translator-credits") ;
 	if ( str_credits != "translator-credits" )
 		dialog .set_translator_credits( str_credits ) ;
@@ -1316,7 +1377,7 @@ void Win_GParted::activate_paste()
 		{
 			Dialog_Partition_Copy dialog( gparted_core .get_fs( copied_partition .filesystem ),
 						      devices[ current_device ] .cylsize ) ;
-			//we don't need the messages/mountpoints of the source partition.
+			//we don't need the messages/mount points of the source partition.
 			copied_partition .messages .clear() ;
 			copied_partition .clear_mountpoints() ;
 			dialog .Set_Data( selected_partition, copied_partition ) ;
@@ -1340,6 +1401,7 @@ void Win_GParted::activate_paste()
 	{
 		Partition partition_new = selected_partition ;
 		partition_new .filesystem = copied_partition .filesystem ;
+		partition_new .label = copied_partition .label ;
 		partition_new .color = copied_partition .color ;
 		partition_new .set_used( copied_partition .sectors_used ) ;
 		partition_new .messages .clear() ;
@@ -1496,7 +1558,7 @@ void Win_GParted::activate_format( GParted::FILESYSTEM new_fs )
 	     fs .MAX && selected_partition .get_length() > fs .MAX ) 
 	{
 		Gtk::MessageDialog dialog( *this,
-					   String::ucompose( _("Cannot format this filesystem to %1."),
+					   String::ucompose( _("Cannot format this file system to %1."),
 						   	     Utils::get_filesystem_string( new_fs ) ) ,
 					   false,
 					   Gtk::MESSAGE_ERROR,
@@ -1505,12 +1567,12 @@ void Win_GParted::activate_format( GParted::FILESYSTEM new_fs )
 
 		if ( selected_partition .get_length() < fs .MIN )
 			dialog .set_secondary_text( String::ucompose( 
-						_( "A %1 filesystem requires a partition of at least %2."),
+						_( "A %1 file system requires a partition of at least %2."),
 						Utils::get_filesystem_string( new_fs ),
 						Utils::format_size( fs .MIN ) ) );
 		else
 			dialog .set_secondary_text( String::ucompose( 
-						_( "A partition with a %1 filesystem has a maximum size of %2."),
+						_( "A partition with a %1 file system has a maximum size of %2."),
 						Utils::get_filesystem_string( new_fs ),
 						Utils::format_size( fs .MAX ) ) );
 		
@@ -1533,7 +1595,7 @@ void Win_GParted::activate_format( GParted::FILESYSTEM new_fs )
 	part_temp .status = GParted::STAT_FORMATTED ;
 	
 	//if selected_partition is NEW we simply remove the NEW operation from the list and
-	//add it again with the new filesystem
+	//add it again with the new file system
 	if ( selected_partition .status == GParted::STAT_NEW )
 	{
 		//remove operation which creates this partition
@@ -1595,9 +1657,9 @@ void Win_GParted::thread_unmount_partition( bool * succes, Glib::ustring * error
 	if ( *succes && failed_mountpoints .size() )
 	{
 		*succes = false ;
-		*error = _("The partition could not be unmounted from the following mountpoints:") ;
+		*error = _("The partition could not be unmounted from the following mount points:") ;
 		*error += "\n\n<i>" + Glib::build_path( "\n", failed_mountpoints ) + "</i>\n\n" ;
-		*error +=  _("Most likely other partitions are also mounted on these mountpoints. You are advised to unmount them manually.") ;
+		*error +=  _("Most likely other partitions are also mounted on these mount points. You are advised to unmount them manually.") ;
 	}
 	else
 		*error = "<i>" + Glib::build_path( "\n", errors ) + "</i>" ;
@@ -1856,7 +1918,7 @@ void Win_GParted::activate_apply()
 				   Gtk::BUTTONS_NONE,
 				   true );
 	dialog .set_secondary_text( _( "It is recommended to backup valuable data before proceeding.") ) ;
-	dialog .set_title( _( "Apply operations to harddisk" ) );
+	dialog .set_title( _( "Apply operations to device" ) );
 	
 	dialog .add_button( Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL );
 	dialog .add_button( Gtk::Stock::APPLY, Gtk::RESPONSE_OK );
